# metadata fields
id_string = "id"
calculated_string = "calculated"
frequency_string = "frequency"
min_value_string = "minValue"
max_value_string = "maxValue"
start_time_string = "startTime"
is_integer_string = "isInteger"
delay_string = "delay"

# frequencies
instant_string = "instant"
daily_string = "daily"
weekly_string = "weekly"
monthly_string = "monthly"
quarterly_string = "quarterly"
annually_string = "annually"

frequencies = [instant_string, daily_string, weekly_string, monthly_string, quarterly_string, annually_string]
